from toil.common import Toil
from toil.job import Job
from collections import OrderedDict

import pwd
import getpass
import argparse
import os
import sys
import json
import docker
import itertools
import time
from runner.runner import docker_runner, decorator_wrapt, docker_runner_method


class Annotation(Job):

    def __init__(self, inputfile, outpath, sampleName, image, volumes, *args, **kwargs):
        self.inputfile = inputfile
        self.outpath = outpath
        self.sampleName = sampleName
        self.image = image
        self.volumes = volumes

        self.commandLine = ['bash', '/usr/local/bin/OncoDecode.sh',
                            self.inputfile, self.outpath, self.sampleName]
        super(Annotation, self).__init__(*args, **kwargs)

    def run(self, fileStore):
        client = docker.from_env()
        lines = open(self.inputfile, 'r').readlines()
        l_n = 0
        for line in lines:
            if line[0] == "#":
                continue
            else:
                l_n = l_n + 1
        if l_n > 0:
            docker_runner_method(self, self.commandLine,
                                 "Annotation", self.image, self.volumes)
        else:
            self.log(self.inputfile +
                     ' has less than 1 mutation pos, so do not run oncodecode')


def cmd_parser(absPath):
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument('--inputfile', help="vcf mutation file"),
    parser.add_argument('--outpath', help='The output folder'),
    parser.add_argument('--sampleName', help='sample name'),
    parser.add_argument('--version', type=argparse.FileType('r'))
    parser.add_argument('--volumes', type=argparse.FileType('r'))
    return parser.parse_args()


def vcf_oncodecode_fun(root, inputfile, outpath, sampleName, version, volumes):
    job = Annotation(inputfile, outpath, sampleName,
                     version['vcf_oncodecode'], volumes)
    root.addChild(job)


def run_vcf_oncodecode(root, outpath, group_name, variants, models, mutantTypes, strategy, version, volumes):
    for variant, model, mutantType in itertools.product(variants, models, mutantTypes):
        if model == 'xgboost' and variant == 'snv' and mutantType == 'Somatic':
            inputfile = outpath + '/' + group_name + '.xgboost.' + \
                strategy + '.' + mutantType + '.' + variant + '.pass.vcf'
            job = Annotation(inputfile, outpath, group_name,
                             version['vcf_oncodecode'], volumes)
            root.addChild(job)


if __name__ == "__main__":
    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    options = cmd_parser(absPath)
    inputfile = options.inputfile
    outpath = options.outpath
    sampleName = options.sampleName
    version = json.load(options.version, object_pairs_hook=OrderedDict)
    volumes = json.load(options.volumes, object_pairs_hook=OrderedDict)
    start_job = Job()
    vcf_oncodecode_fun(start_job, inputfile, outpath,
                       sampleName, version, volumes)

    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()



from toil.common import Toil
from toil.job import Job

import pwd
import getpass
import argparse
import subprocess
import commands
import pysam
import os
import sys
import json
import docker
import time
import re
import variant_interface
from runner.runner import docker_runner, decorator_wrapt


class GVC(Job):
    @decorator_wrapt
    def __init__(self, input_bam, intput_ref, output_path,  image, volumes, variants, reg=None, freg=None, dbsnp=None, ccd=None, bed=None, *args, **kwargs):
        self.image = image
        self.volumes = volumes
        self.commandLine = ['gvc', '-f', intput_ref]
        if isinstance(input_bam, list):
            for bam in input_bam:
                self.commandLine.extend(['--in', bam])
        else:
            self.commandLine.extend(['--in', self.input_bam])
        if reg and freg:
            self.commandLine.extend(
                ['--out', output_path + '.' + freg, '--region', reg, '--fregion', freg])
        else:
            self.commandLine.extend(
                ['--out', output_path])
        if dbsnp:
            self.commandLine.extend(['--dbsnp', dbsnp])
        if ccd:
            self.commandLine.extend(['--ccd', ccd])
        if bed:
            self.commandLine.extend(['--bed', bed])
        for variant in variants:
            if variant == 'snv':
                self.commandLine.extend(['--snp'])
            elif variant == 'indel':
                self.commandLine.extend(['--indel'])
            elif variant == 'cnv':
                self.commandLine.extend(['--ghs', '--cov'])
            else:
                self.commandLine.extend(['--sv'])
        super(GVC, self).__init__(*args, **kwargs)

    @docker_runner('GVC')
    def run(self, fileStore):
        return self.commandLine


class catJob(Job):

    def __init__(self, intputList, outputName, rm_gvcs=False, *args, **kwargs):
        self.intputList = intputList
        self.rm_gvcs = rm_gvcs
        self.commandLine = ['cat']
        self.commandLine.extend(self.intputList)
        self.commandLine.extend(['>', outputName])
        self.outputfile = outputName
        super(catJob, self).__init__(*args, **kwargs)

    def run(self, fileStore):
        commands.getstatusoutput('\t'.join(self.commandLine))
        if self.rm_gvcs:
            for path in self.intputList:
                os.remove(path)

    def get_all_variant(self):
        return self.outputfile


def read_ccd(ccd_name):
    with open(ccd_name) as F:
        return map(lambda x: x.strip('\n').strip('\r').split('\t')[:3], F)


def filter_bed(ccd_regions, limit_size):
    return filter(lambda x: len(x[0]) < limit_size, ccd_regions)


def filter_references(references, reference_lengths, limit_size):
    l_filter_references = list()
    l_filter_reference_lengths = list()
    for i_ref, i_length in zip(references, reference_lengths):
        if len(i_ref) < limit_size:
            l_filter_references.append(i_ref)
            l_filter_reference_lengths.append(i_length)
    return l_filter_references, l_filter_reference_lengths


def split_reference(references, reference_lengths, segmentSize):
    list_intervals = list()
    for reference, reference_length in zip(references, reference_lengths):
        list_intervals.extend(map(lambda x: [
                              reference, x, x+segmentSize-1], range(1, reference_length, segmentSize)))
        if list_intervals[-1][2] > reference_length:
            list_intervals[-1][2] = reference_length
    return list_intervals


def split_ccd(ccd_region, segmentSize):
    regions = [[ccd_region[0][0], int(
        ccd_region[0][1]), int(ccd_region[0][2])]]
    for chrom, start, end in ccd_region[1:]:
        if chrom != regions[-1][0] or regions[-1][1] + segmentSize < int(end):
            regions.append([chrom, int(start), int(end)])
        else:
            regions[-1][2] = int(end)
    return regions


def get_regions(ccd_name, references, lengths, segmentSize, limit_regions=None):
    segmentSize = int(segmentSize)
    origin_regions = list()
    if ccd_name:
        origin_region = read_ccd(ccd_name)
        origin_regions = split_ccd(origin_region, segmentSize)
    else:
        origin_regions = split_reference(references, lengths, segmentSize)
    return filter(lambda x: limit_regions is None or x[0] in limit_regions, origin_regions)


def get_regions_file(reginos_file):
    if reginos_file is None:
        return None
    with open(reginos_file) as F:
        try:
            return map(lambda x: re.search('\s?(\S+)\S?', x).group(1), F)
        except Exception as E:
            print ("Regions Format Error")
            sys.exit(-1)


def gvc_pipeline(root, reference_filename, segmentSize, bam, outpath, sampleName, version, volumes, variants, rm_bam=False, rm_gvcs=True, dbsnp=None, ccd=None, limit_regions=None):
    Fasta = pysam.Fastafile(reference_filename)
    references = Fasta.references

    if isinstance(list,bam) and len(bam) > 1 :
        memory = '16G'
    else:
        memory =  '8G'
    if segmentSize:
        region = get_regions(ccd, Fasta.references,
                             Fasta.lengths, segmentSize, limit_regions)

        if ccd:
            gvc_jobs = map(lambda x: GVC(bam, reference_filename,
                                         outpath + '/' + sampleName, version['gvc'], volumes, variants, str(
                                             x[0]) + ":" + str(x[1]) + "-" + str(x[2]),
                                         str(x[0]) + ":" + str(x[1]) + "-" + str(x[2]), dbsnp=dbsnp, ccd=ccd, bed=ccd, memory=memory), region)
        else:
            gvc_jobs = map(lambda x: GVC(bam, reference_filename,
                                         outpath + '/' +
                                         sampleName, version['gvc'], volumes, variants,
                                         str(x[0]) + ":" + str(x[1]) +
                                         "-" + str(x[2] + 2500),
                                         str(x[0]) + ":" + str(x[1]) +
                                         "-" + str(x[2]),
                                         dbsnp=dbsnp, ccd=ccd, bed=ccd, memory=memory), region)
        map(lambda job: root.addChild(job), gvc_jobs)
    else:
        gvc_jobs = GVC(bam, reference_filename, outpath + '/' + sampleName,
                       version['gvc'], volumes, variants, dbsnp=dbsnp, ccd=ccd, bed=ccd, memory=memory)
        root.addChild(gvc_jobs)
    newmethod54(segmentSize, variants, outpath, sampleName, region, rm_gvcs, root)
    Fasta.close()
    vi = variant_interface.VariantFeature()
    vi.set_feature(
        "snv", outpath + '.'.join([sampleName, 'pup']))
    vi.set_feature(
        "indel", outpath + '.'.join([sampleName, 'idf']))
    vi.set_feature(
        "sv", outpath + '.'.join([sampleName, 'svf']))
    return {"snv": outpath + '.'.join([sampleName, 'pup']),
            "indel": outpath + '.'.join([sampleName, 'idf']),
            "sv": outpath + '.'.join([sampleName, 'svf']),
            "cnv": [outpath+'.'.join([sampleName, 'ghs']), outpath+'.'.join([sampleName, 'cov'])]}

def newmethod54(segmentSize, variants, outpath, sampleName, region, rm_gvcs, root):
    if segmentSize:
        datatype = []
        for variant in variants:
            if variant == 'snv':
                datatype = datatype+['pup']
            elif variant == 'indel':
                datatype = datatype + ['idf']
            elif variant == 'sv':
                datatype = datatype + ['svf']
            elif variant == 'cnv':
                datatype = datatype + ['ghs', 'cov']
        cat_jobs = map(lambda suffix: catJob(
            map(lambda x: outpath + '.'.join([sampleName, str(x[0]) + ":" + str(x[1]) + "-" + str(x[2]), suffix]),
                region), outpath + '/' + '.'.join([sampleName, suffix]), rm_gvcs), datatype)
        map(lambda catPupJob: root.addFollowOn(catPupJob), cat_jobs)



def newmethod324():
    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument('bam', nargs='+', help='input bams')
    parser.add_argument('ref', help='references')
    parser.add_argument('seg', help='segmentSize')
    parser.add_argument('--ccd', help='ccd')
    parser.add_argument('outpath', help='output path')
    parser.add_argument('sampleName', help='sample name')
    parser.add_argument('--version', type=argparse.FileType('r'))
    parser.add_argument('--volumes', type=argparse.FileType('r'))
    return parser

if __name__ == "__main__":
    parser = newmethod324()

    options = parser.parse_args()
    version = json.load(options.version)
    volumes = json.load(options.volumes)
    variants = ['cnv', 'indel', 'sv', 'snv']
    start_job = Job()
    gvc_pipeline(start_job, options.ref, options.seg, options.bam, options.outpath,
                 options.sampleName, version, volumes, variants, ccd=options.ccd)

    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()



from toil.common import Toil
from toil.job import Job

import pwd
import getpass
import docker
import functools
import itertools
import commands
import os
import time
import re
import json
import shutil
from collections import OrderedDict
from collections import defaultdict

from runner.runner import docker_runner, decorator_wrapt

import variant_interface

class MarkErrorPattern(Job):

    @decorator_wrapt
    def __init__(self, filter_file, xgboost_file, tumor_bam ,reference ,output_file ,sampleID ,variant, image, volumes,*args,**kwargs):
        self.image = image
        self.volumes = volumes
        self.commandLine = ['sh','/bin/tr_hdr.sh',filter_file,xgboost_file,tumor_bam,reference,output_file,sampleID,variant.upper()]
        super(MarkErrorPattern,self).__init__(*args,**kwargs)

    @docker_runner('MarkErrorPattern')
    def run(self, fileStore):
        return self.commandLine

class LinearPredict(Job):

    @decorator_wrapt
    def __init__(self, inputfile, outfile, json_file, model, image, volumes, *args,
                 **kwargs):
        self.inputfile = inputfile
        self.outfile = outfile
        self.setting_json = json_file
        self.model = model
        self.image = image
        self.volumes = volumes
        self.commandLine = [self.setting_json,
                            self.model, self.inputfile, self.outfile]
        #print(' '.join(self.commandLine))
        super(LinearPredict, self).__init__(*args, **kwargs)

    @docker_runner('LinearPredict')
    def run(self, fileStore):
        return self.commandLine


class XGBoostPredict(Job):

    @decorator_wrapt
    def __init__(self, inputfile, outfile, json_file, xml, modelPath, image, volumes, threads='1', *args, **kwargs):
        self.commandLine = ['gvc_pre.py', inputfile,
                            outfile, json_file, xml, modelPath]

        self.image = image
        self.volumes = volumes
        super(XGBoostPredict, self).__init__(*args, **kwargs)

    @docker_runner('XGBoostPredict')
    def run(self, fileStore):
        return self.commandLine


class BedFilter(Job):

    @decorator_wrapt
    def __init__(self, variant, mutantType, inputfile, bed, bam,image, volumes, *args, **kwargs):
        self.variant = variant
        self.mutantType = mutantType
        self.inputfile = inputfile
        self.bed = bed
        self.image = image
        self.volumes = volumes

        self.commandLine = ['/usr/bin/filter.sh',
                            inputfile, bed, mutantType + '_' + variant,bam]
        super(BedFilter, self).__init__(*args, **kwargs)

    @docker_runner('BedFilter')
    def run(self, fileStore):
        return self.commandLine


choice_vcf_dict = {
    'Somatic': {
        'snv': 'pup2vcf-somatic.pl',
        'indel': 'idf2vcf-somatic.pl',
        'sv': 'sv2vcf-somatic.pl'
    },
    'Germline': {
        'snv': 'pup2vcf-germline.pl',
        'indel': 'idf2vcf-germline.pl',
        'sv': 'sv2vcf-germline.pl'
    }
}
class PostProcessVCF(Job):
    @decorator_wrapt
    def __init__(self,variant, mutantType,inputfile,sample_name,output_vcf,bam,image, volumes, *args, **kwargs):
        self.image = image
        self.volumes = volumes
        self.commandLine = ['gvc2vcf.py',variant,mutantType,sample_name,inputfile,output_vcf,bam]
        super(PostProcessVCF,self).__init__(*args, **kwargs)

    @docker_runner('PostProcessVCF')
    def run(self, fileStore):
        return self.commandLine
    
class GenerateVCF(Job):
    @decorator_wrapt
    def __init__(self, variant, mutantType, inputfile, vcf_file_name, group_name, image, volumes,*args, **kwargs):
        self.variant = variant
        self.mutantType = mutantType
        self.inputfile = inputfile
        self.vcf_file_name = vcf_file_name
        self.group_name = group_name
        self.image = image
        self.volumes = volumes
        self.commandLine = [choice_vcf_dict[self.mutantType]
                            [self.variant], group_name, inputfile, vcf_file_name]
        super(GenerateVCF, self).__init__(*args, **kwargs)

    @docker_runner('GenerateVCF')
    def run(self, fileStore):
        return self.commandLine

class tmp_rm(Job):
    def __init__(self, group_name, outputPath, variants, models, strategy, mutantTypes, rm_predict, bed=None, *args, **kwargs):
        self.group_name = group_name
        self.outputPath = outputPath
        self.variants = variants
        self.models = models
        self.strategy = strategy
        self.mutantTypes = mutantTypes
        self.rm_predict = rm_predict
        self.bed = bed
        super(tmp_rm, self).__init__(*args, **kwargs)

    def run(self, fileStore):
        shutil.rmtree(os.path.join(self.outputPath, 'tmp'))
        if self.bed:
            os.remove(self.outputPath + '/' +
                      os.path.basename(self.bed) + '.sort')
            os.remove(self.outputPath + '/' +
                      os.path.basename(self.bed) + '.sort.reg')


def post_filter(root,predict_dict,variants,reference,tumor_bam,sample_name,output_path,version,volumes):
    #print (predict_dict)
    vd = variant_interface.VariantInterface()
    for variant in variants:
        MarkErrorPath =  os.path.join(os.path.dirname(predict_dict[variant]['filter']),'.'.join([sample_name,'errorpattern',variant]))
        if 'xgboost' in  predict_dict[variant].keys():
            markJob = MarkErrorPattern(predict_dict[variant]['filter'],predict_dict[variant]['xgboost'],tumor_bam,reference,MarkErrorPath,sample_name,variant,version['postgvc'], volumes)
        else:
            markJob = MarkErrorPattern(predict_dict[variant]['filter'],predict_dict[variant]['grig'],tumor_bam,reference,MarkErrorPath,sample_name,variant,version['postgvc'], volumes)
        root.addChild(markJob)
        vcfPath = os.path.join(output_path,'.'.join([sample_name,'Somatic',variant,'vcf']))
        postJob = PostProcessVCF(variant,'Somatic',MarkErrorPath,sample_name,vcfPath,tumor_bam,version['gvc2vcf'],volumes)
        vd.set_vcf(variant,'Somatic',vcfPath)
        markJob.addChild(postJob)

def Predict_pipeline(root, group_name, sampleList, outputPath, infoDict, inputfileDict, variants, sequenceTypes, strategy, mutantTypes, bed, models, xml, modelPath, bam,version, volumes):

    predict_dict = defaultdict(type(defaultdict()))
    # print variants, models, mutantTypes
    for variant, model, mutantType in itertools.product(variants, models, mutantTypes):
        # print simpleName
        inputfile = inputfileDict[variant]
        dirname = os.path.dirname(inputfile)
        json_file = infoDict[variant][mutantType]
        sample_name = '.'.join(
            [group_name, model, strategy, mutantType, variant])
        filter_path = dirname + '/' + sample_name
        output_sample_name = outputPath + '/' + sample_name

        predictJob = None
        if model == 'filter' and mutantType == 'Germline':
            continue
        # and not(variant == 'indel' and mutantType == 'Somatic')
        if model == 'xgboost' and (variant != 'sv'):
            predictJob = XGBoostPredict(inputfile, filter_path, json_file, xml,
                                        modelPath, version['xgboost'], volumes, cores='10', memory="20G")
        if model != 'xgboost':
            predictJob = LinearPredict(inputfile, filter_path, json_file, model,
                                       version['filter'], volumes)

        if predictJob is None:
            continue
        root.addChild(predictJob)

        if model == 'filter' and mutantType == 'Somatic':
            vcf_file_name = os.path.join(outputPath,'.'.join([group_name,mutantType, variant,'all','vcf']))
        else:
            vcf_file_name = os.path.join(outputPath,'.'.join([group_name,mutantType, variant,'pass','vcf']))
        if strategy == 'WES' or strategy == 'Panel_tissue':
            # bed filter
            bed_filterJob = BedFilter(
                variant, mutantType, filter_path, bed,bam, version['bed_filter'], volumes)
            predictJob.addChild(bed_filterJob)
            # TODO: After fixed BUGs(Indel post-process) , variant in ['snv','indel']
            if mutantType == 'Somatic' and  variant != 'sv':
                predict_dict[variant][model] = filter_path + ".bed.filter"
            else:
                vcfJob = GenerateVCF(variant, mutantType, filter_path + ".bed.filter",
                    vcf_file_name, group_name, version['gvc2vcf'], volumes)
                
                if model != 'filter':
                    vd = variant_interface.VariantInterface()
                    vd.set_vcf(variant,mutantType,vcf_file_name)
                bed_filterJob.addChild(vcfJob)       
        elif strategy == 'WGS':
            if mutantType == 'Somatic' and  variant != 'sv':
                predict_dict[variant][model] = filter_path
            else:
                vcfJob = GenerateVCF(variant, mutantType, filter_path, vcf_file_name, group_name,
                                        version['gvc2vcf'], volumes)
                predictJob.addChild(vcfJob)
                if model != 'filter':
                    vd = variant_interface.VariantInterface()
                    vd.set_vcf(variant,mutantType,vcf_file_name)
    return predict_dict

def tmp_rm_def(root, group_name, outputPath, variants, models, strategy, mutantTypes, rm_predict, bed):
    tmp_rmJob = tmp_rm(group_name, outputPath, variants,
                       models, strategy, mutantTypes, rm_predict, bed)
    root.addChild(tmp_rmJob)


if __name__ == "__main__":

    absPath = os.path.abspath(os.path.dirname(__file__))
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument('sample_name')
    parser.add_argument('input_dir')
    parser.add_argument('reference')
    parser.add_argument('bam')
    parser.add_argument('output')
    parser.add_argument(
        '--gvc_lib',
        help="The library folder has configuration file." + '\n' +
             'The docker volume file needs to be modified. ' +
             'A dictionary to configure volumes mounted inside the container. ' +
             'The key is either the host path or a volume name, ' +
             'and the value is a dictionary with the keys:' + '\n' +
             'bind: The path to mount the volume inside the container' +
             '(the host path need same with the container path).' + '\n' +
             'mode: rw to mount the volume read/write.' + '\n' +
             'eg: {"/disk": {"bind": "/disk","mode": "rw"}}"',
        default=absPath + '/../gvc_lib/')
    options = parser.parse_args()


    input_gvcs = dict()
    variant = set()
    infoDict = defaultdict(type(defaultdict()))
    mutantTypes = set()
    for filename in os.listdir(options.input_dir):
        if re.search("\.json$", filename):
            with open(os.path.join(options.input_dir, filename)) as F:
                setting = json.load(F)
                if 'variant' in setting:
                    infoDict[setting['variant']][setting['mutantTypes']
                                                 ] = os.path.join(options.input_dir, filename)
                    variant.add(setting['variant'])
                    sequenceTypes = setting['sequenceType']
                    strategy = setting['strategy']
                    mutantTypes.add(setting['mutantTypes'])
        re_gvcs = re.search(options.sample_name+'.(idf|pup|svf)', filename)

        if re_gvcs:

            if re_gvcs.group(1) == 'idf':
                input_gvcs['indel'] = os.path.join(options.input_dir, filename)
            elif re_gvcs.group(1) == 'svf':
                input_gvcs['sv'] = os.path.join(options.input_dir, filename)
            else:
                input_gvcs['snv'] = os.path.join(options.input_dir, filename)

    for filename in os.listdir(os.path.dirname(options.input_dir)):
        if re.search('bed', filename):
            bed = os.path.join(os.path.dirname(options.input_dir), filename)

    gvc_lib = options.gvc_lib
    volumes = {
        "/disk": {
            "bind": "/disk",
            "mode": "rw"
        }
    }
    print("****** loading paramter from template files ******")
    print(sequenceTypes, strategy)
    print(input_gvcs)
    print(infoDict)
    print(variant)
    print(bed)
    print(gvc_lib)

    models = ['filter','grig','xgboost']
    version_path = gvc_lib + '/version.json'
    volumes_path = gvc_lib + '/volumes.json'
    thread_path = gvc_lib + '/thread.json'
    xml_path = gvc_lib + '/format.xml'
    model_path = gvc_lib + '/Model/PE150'

    with open(volumes_path, 'r') as f:
        volumes = json.load(f, object_pairs_hook=OrderedDict)

    with open(version_path, 'r') as verf:
        version = json.load(verf)

    with open(thread_path, 'r') as thref:
        thread = json.load(thref)

    start_job = Job()

    dummy_job = Job()

    outputPath = options.output
    predict_dict = Predict_pipeline(start_job,options.sample_name,['N','T'],outputPath,infoDict,input_gvcs,variant,sequenceTypes,strategy,mutantTypes,bed,models,xml_path,model_path,options.bam,version,volumes)
    start_job = start_job.encapsulate()
    post_filter(start_job,predict_dict,['snv'],options.reference,options.bam,options.sample_name,outputPath,version,volumes)
    
    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()


from collections import defaultdict


class Singleton(object):
    _instance = None

    def __new__(cls, *args, **kw):
        if not cls._instance:
            cls._instance = super(Singleton, cls).__new__(cls, *args, **kw)
        return cls._instance


class VariantInterface(Singleton):
    VariantDict = defaultdict(type(defaultdict()))

    def set_vcf(self, variant, mutantType, vcf):
        self.VariantDict[variant][mutantType] = vcf

    def set_cnv(self, *args):
        self.VariantDict['cnv'] = list(args)

    def get_variants(self):
        return self.VariantDict


class VariantFeature(Singleton):
    VariantDict = dict()

    def set_feature(self, variant, filename):
        self.VariantDict[variant] = filename

    def get_feature(self, variant):
        return self.VariantDict[variant]



import unittest
import pprint

import gvc_vcf_pipeline

pp = pprint.PrettyPrinter()
class TEST_VOLUMES(unittest.TestCase):
    maxDiff=None
    def test_single(self):
        volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/db/ref/')
        self.assertDictEqual(volume,{'/disk':{
            'bind':'/disk',
            "mode": "rw"
        }})

    # def test_file(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/db/ref/human.fa')
    #     self.assertDictEqual(volume,{'/disk/db/ref':{
    #         'bind':'/disk/db/ref',
    #         "mode": "rw"
    #     }})

    # def  test_repeat_files(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/db/ref/human.fa','/disk/db/ref/')
    #     self.assertDictEqual(volume,{'/disk/db/ref':{
    #         'bind':'/disk/db/ref',
    #         "mode": "rw"
    #     }})

    # def  test_link_files_case0(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/db/ref/human.fa','/disk/db/ref/','/disk/yinlh/docker3.0/gvc_lib/Model/PE100/Depth/')
    #     re_volume = {
    #         '/disk/db/ref':{
    #             'bind':'/disk/db/ref',
    #             "mode": "rw"
    #         },
    #         '/disk/yinlh/docker3.0/gvc_lib/Model/PE100/Depth':    
    #         {
    #             'bind':'/disk/yinlh/docker3.0/gvc_lib/Model/PE100/Depth',
    #             'mode': 'rw'
    #         },
    #         '/disk/gvcgroup/zhangzb/Model/v3.0/gvc_lib/Model_idcs/PE100/Depth':
    #         {
    #             'bind':'/disk/gvcgroup/zhangzb/Model/v3.0/gvc_lib/Model_idcs/PE100/Depth',
    #             'mode':'rw'
    #         }
    #     }
    #     #pp.pprint(volume)
    #     #pp.pprint(re_volume)
    #     self.assertDictEqual(volume,re_volume)

    # def test_link_files_case1(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/yinlh/db/ref/')
    #     ret_volumes = {
    #         '/disk/db/ref':{
    #             'bind':'/disk/db/ref',
    #             "mode": "rw"
    #         },
    #         '/disk/yinlh/db/ref/':{
    #             'bind':'/disk/yinlh/db/ref/',
    #             "mode": "rw"
    #         },
    #      }

    # def test_link_files_case2(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/yinlh/db/ref/human.fa')
    #     ret_volumes = {
    #         '/disk/db/ref':{
    #             'bind':'/disk/db/ref',
    #             "mode": "rw"
    #         },
    #         '/disk/yinlh/db/ref/':{
    #             'bind':'/disk/yinlh/db/ref/',
    #             "mode": "rw"
    #         },
    #      }

    # def test_relate_files(self):
    #     volume = gvc_vcf_pipeline.get_dynamic_volumes('/disk/yinlh/gvc_pre/../db/ref/human.fa')
    #     ret_volumes = {
    #         '/disk/db/ref':{
    #             'bind':'/disk/db/ref',
    #             "mode": "rw"
    #         },
    #         '/disk/yinlh/db/ref/':{
    #             'bind':'/disk/yinlh/db/ref/',
    #             "mode": "rw"
    #         },
    #      }       


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
import sys
import argparse
import pysam

def cmdPhase():
    parser = argparse.ArgumentParser(description = 'load and check Bam')
    parser.add_argument('reference',help='reference filename ')
    parser.add_argument('input_bam', help='input bed filename ')
    return parser.parse_args()

# check chr of bam and ref
def bamRefCheck( refFn , bamFn):
   # print refFn,bamFn
    bamFile = pysam.AlignmentFile(bamFn)
    #print bamFile.references
    fastaFile = pysam.FastaFile(refFn)
   # print fastaFile.references
   # print "bam:",bamFile.references
   # print "header",fastaFile.references
    return tuple(bamFile.references) ==  tuple(fastaFile.references)
# check bam.bai
def bamBai(bamFn):
    try:
        f = open(bamFn + '.bai')
        f.close()
    except Exception as E:
        print E
        sys.exit(-1)
if __name__ == '__main__':
    args = cmdPhase()
    bamRefCheck(args.reference, args.input_bam)
    bamBai(args.input_bam)



#!/usr/bin/python
import sys
import argparse
import pysam


def cmdPhase():
    parser = argparse.ArgumentParser(description='Sort BedFile and Get BedReg')
    parser.add_argument('reference', help='reference filename ')
    parser.add_argument('input_bed', help='input bed filename ')
    parser.add_argument('output_bed', help='output bed filename')
    return parser.parse_args()


def loadReference(filename):
    try:
        fasta = pysam.FastaFile(filename)
    except ValueError as E:
        print 'if index file is missing'
        sys.exit(-1)
    except IOError as E:
        print 'if file could not be opened ', filename
        sys.exit(-1)
    return fasta


def fetchBedLines(filename, reference_name):
    bedLines = []
    with open(filename) as F:
        for line in F:
            Lines = line.strip('\n').split('\t')
            if len(Lines) < 3:
                print('Bed file format error. Please make sure the bed file is tab delimited.\
                     For more info on bed file format, please check out {}'.format(filename))
                sys.exit(-1)
            if Lines[0] not in reference_name:
                #Lines[0] = Lines[0].replace('chr','')
                # if Lines[0]  not in reference_name :
                print ('Chromosome name in bed file is different from reference.\
                    Check {} for correct chromosome names.'.format(filename))
                sys.exit(-1)
            Lines = Lines[0:3]
            bedLines.append(Lines)
    return bedLines


def bedSort_run(reference, input_bed, output_bed):
    fasta = loadReference(reference)
    references = dict((value, key)
                      for key, value in enumerate(fasta.references))

    with open(output_bed, 'w') as F:
        for item in sorted(fetchBedLines(input_bed, fasta.references), key=lambda x: (references[x[0]], int(x[1]))):
            F.write('\t'.join(item)+'\n')
    return None


def write_bed_reg(bedFile, outputFile):
    with open(bedFile, 'r') as F:
        chr_sum = []
        chr_name = []
        for bedline in F:
            dataList = bedline.strip('\n').split('\t')
            if len(chr_name) == 0:
                sum_base = int(dataList[2]) - int(dataList[1])
                chr_name.append(dataList[0])
            elif chr_name[-1] != dataList[0]:
                chr_sum.append(sum_base)
                sum_base = int(dataList[2]) - int(dataList[1])
                chr_name.append(dataList[0])
            else:
                sum_base = sum_base + int(dataList[2]) - int(dataList[1])
        chr_sum.append(sum_base)
    # print outputFile
    with open(outputFile, 'w') as F:
        F.write('#1.0-5-gd74ff94' + '\n')
        for item in zip(chr_name, chr_sum):
            F.write(item[0] + '\t' + str(item[1]) + '\n')
# check bed, sort bed and get bed.reg


def bed_check_sort_reg(ref, input_bed, sort_bed):
    fasta = loadReference(ref)
    references = dict((value, key)
                      for key, value in enumerate(fasta.references))
    with open(sort_bed, 'w') as F:
        for item in sorted(fetchBedLines(input_bed, fasta.references),
                           key=lambda x: (references[x[0]], int(x[1]))):
            F.write('\t'.join(item) + '\n')
    write_bed_reg(sort_bed, sort_bed + '.reg')


if __name__ == '__main__':

    args = cmdPhase()
    #fasta = loadReference(args.reference)

    #references = dict((value,key) for key,value in enumerate(fasta.references))
    # with open(args.output_bed,'w') as F:
    #    for item  in  sorted(fetchBedLines(args.input_bed,fasta.references),key=lambda x:(references[x[0]], int(x[1]))):
    #        F.write('\t'.join(item)+'\n')
    #write_bed_reg(args.output_bed, args.output_bed + '.reg')
    bed_check_sort_reg(args.reference, args.input_bed, args.output_bed)




from toil.common import Toil
from toil.job import Job

import os
import shutil
import docker

import itertools
import variant_interface


from runner.runner import docker_runner, decorator_wrapt, docker_runner_method


class cnv(Job):

    def __init__(self, group_name, input_ghs, input_cov, outpath, sample_list, input_dirname, image, volumes, enable_panel, *args, **kwargs):
        sample_names = ','.join(sample_list[1:])
        self.image = image
        self.volumes = volumes
        if enable_panel:
            self.commandLine = ['sh', '/usr/local/bin/Panel/GVC_CNV_R.sh', input_ghs,
                                input_cov, sample_names, input_dirname]
        else:
            self.commandLine = ['GVC_CNV_R.sh', input_ghs,
                                input_cov, sample_names, input_dirname]
        self.sample_list = sample_list[1:]
        #self.suffixes = ['.CBS', '.cnv.bandAnn.xls', '.cnv.geneAnn.xls',
        #                 '.cnv.xls', '.correctedMutAF.LRR.pdf']
        self.suffixes = ['.cnv.geneAnn.xls']
        self.outpath = outpath
        self.input_dirname = input_dirname
        self.group_name = group_name
        self.suffixes_output = False if enable_panel else True
        super(cnv, self).__init__(*args, **kwargs)
        vd = variant_interface.VariantInterface()
        if  enable_panel :
            vd.set_cnv(os.path.join(self.outpath,self.group_name+'.cnv.simp'))
        else:
            vd.set_cnv(os.path.join(self.outpath,self.group_name+'.cnv.simp'),os.path.join(self.outpath,group_name+self.suffixes[0]))

    def run(self, fileStore):
        client = docker.from_env()
        docker_runner_method(self, self.commandLine,
                             "cnv", self.image, self.volumes)

        if self.suffixes_output:
            for  suffix in self.suffixes:
                sample_name = self.sample_list[0]
                shutil.move(os.path.join(self.input_dirname,
                                         sample_name+suffix), os.path.join(self.outpath, self.group_name+suffix))

        for sample_name in self.sample_list:
            shutil.move(self.input_dirname + '/' +
                        sample_name+'.cnv.simp', self.outpath+'/'+self.group_name+'.cnv.simp')


def cmdPhaser():
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument("ghs")
    parser.add_argument("cov")
    parser.add_argument("sample", nargs="+")
    parser.add_argument("input_dirname")
    parser.add_argument("outpath")
    return parser.parse_args()


if __name__ == "__main__":
    options = cmdPhaser()

    volumes = {
        "/disk": {
            "bind": "/disk",
            "mode": "rw"
        }
    }
    image = "cnv:1.0"
    start_job = cnv(options.ghs, options.cov, options.outpath, options.sample,
                    options.input_dirname, image, volumes)

    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()




import unittest


import gvc_pipeline


class TestSplitMethod(unittest.TestCase):

    def test_split_method(self):
        input_regions = [['1',400,500],
                        ['2',5000,6000],
                        ['2',6010,7000]
                        ]
        self.assertListEqual(input_regions,
            gvc_pipeline.split_ccd(input_regions,500))
        input_regions = [['1',400,500],
                        ['2',5000,5020],
                        ['2',5030,5035],
                        ['3',5030,5035]
                        ]
        output_regions = [['1',400,500],
                        ['2',5000,5035],
                        ['3',5030,5035]
                        ]
        self.assertListEqual(output_regions,
            gvc_pipeline.split_ccd(input_regions,500))

    def test_split_method_WGS(self):
        refernces = ['1']
        lens = [23]
        output = [['1',1,5],['1',6,10],['1',11,15],['1',16,20],['1',21,23
        ]]
        self.assertListEqual(output,gvc_pipeline.split_reference(refernces,lens,5))
        refernces = ['1','2']
        lens = [500,600]
        output = [['1',1,500],['2',1,500],['2',501,600]]
        self.assertListEqual(output,gvc_pipeline.split_reference(refernces,lens,500))

    def test_filter_bed(self):
        input_regions = [['1',400,500],
                        ['fake_name_too_long',5000,5020],
                        ['2',5030,5035],
                        ['fake_name_too_long',5030,5035]
                        ]
        output = [['1',400,500],['2',5030,5035]]
        self.assertListEqual(output,gvc_pipeline.filter_bed(input_regions,10))
        int_ref = ['fake_name_too_long','1','fake_name_too_long','2']
        int_lens =[0,1,3,4]
        out_ref ,out_lens = gvc_pipeline.filter_references(int_ref,int_lens,10)
        self.assertListEqual(out_ref,['1','2'])
        self.assertListEqual(out_lens,[1,4])

if __name__ == "__main__":
    unittest.main()


#!/usr/bin/python
import argparse
import sys
import os
import subprocess
import time
import json
from collections import OrderedDict
import argparse


def QC_common_time(runtime, check_type, QC_types=['QC']):
    return map(lambda x: runtime[x][check_type], QC_types)


def QC_time_def(runtime):
    return QC_common_time(runtime, 'total')


def QC_thread_def(runtime):
    return QC_common_time(runtime, 'thread')


def cmdPhase(absPath):
    parser = argparse.ArgumentParser(description="Bam_Vcf")
    parser.add_argument(
        'input_json',
        help='The json file stores names and paths of both normal and tumor samples.'
        + '\n' +
        'eg: {"N": ["/disk/N.sort.dup.bam"], "T": ["/disk/T.sort.dup.bam"]}')
    parser.add_argument('reference', help='The reference fasta file')
    parser.add_argument('outpath', help='The output folder')
    parser.add_argument(
        '--dbsnp',
        help="The Single Nucleotide Polymorphism Database(dbSNP) file", required=True)
    parser.add_argument(
        '--bed',
        help="""BED file for WES or Panel analysis. It should be a TAB delimited file
with at least three columns: chrName, startPosition and endPostion""")
    parser.add_argument(
        '--segmentSize',
        help="Chromosome segment size for each GVC job, set to 100000000 (100MB) or larger for better performance. Default is to run only one GVC job.")
    parser.add_argument(
        '--gvc_lib',
        help="GVC library folder",
        required=True)
    parser.add_argument(
        '--mutantType',
        help="Switch algorithm for germline or somatic mutation detection",
        choices=['Somatic', 'Germline'],
        required=True,
        action='append',
        default=[])
    parser.add_argument(
        '--region',
        help=argparse.SUPPRESS
    )
    parser.add_argument('--strategy', choices=['WES', 'Panel', 'WGS'],
                        help='Switch algorithm for WES, Panel or WGS analysis', required=True)
    parser.add_argument(
        '--sample_name', help="Name of the sample to be analyzed.", required=True)
    parser.add_argument('--CNV', action='store_true', default=False,
                        help='calculate and output CNV simultaneously')
    parser.add_argument('--SV', action='store_true', default=False,
                        help='calculate and output SV simultaneously')
    parser.add_argument(
        '--maxMemory', help="The maximum amount of memory to request from the batch" + '\n' +
        "system at any one time, eg: 32G.")
    parser.add_argument(
        '--maxCores', help="The maximum number of CPU cores to request from the batch" + '\n' +
        "system at any one time, eg: 8.")
    return parser.parse_args()
#


if __name__ == "__main__":
    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    options = cmdPhase(absPath)
    # abspath = os.path.dirname(os.path.abspath(__file__)) + '/'
    commandLine = ['python', '-m', 'gvc4bam.gvc_vcf_pipeline', options.outpath + '/jobStore', options.input_json, options.reference,
                   options.outpath]

    if options.region:
        commandLine.extend(['--region', options.region])

    if options.CNV:
        commandLine.append('--CNV')

    if options.SV:
        commandLine.append('--SV')

    if options.dbsnp:
        commandLine.extend(['--dbsnp', options.dbsnp])
    if options.bed:
        commandLine.extend(['--bed', options.bed])
    if options.segmentSize:
        commandLine.extend(['--segmentSize', options.segmentSize])
    if options.maxMemory:
        commandLine.extend(['--maxMemory', options.maxMemory])
    if options.maxCores:
        commandLine.extend(['--maxCores', options.maxCores])

    for mutantType in options.mutantType:
        commandLine.extend(['--mutantType', mutantType])

    commandLine.extend(['--gvc_lib', options.gvc_lib, '--strategy', options.strategy, '--sample_name', options.sample_name, '--clean=never',
                        '--stats', '--rmtmp'])
    STDERR = open(options.outpath + '/logFile', 'w')
    Start_time = time.time()
    # print "Start_time:", Start_time

    proc1 = subprocess.Popen(commandLine, stderr=STDERR)

    if proc1.wait() != 0:
        with open(options.outpath + '/logFile') as flog:
            sys.stderr.writelines(flog.readlines())
            sys.exit(-3)
    cmd2 = ['python', absPath + '/check_runtime.py', options.outpath + '/logFile', options.outpath + '/jobStore',
            options.outpath + '/runtime.json']
    proc2 = subprocess.Popen(cmd2, stderr=STDERR)
    proc2.wait()
    End_time = time.time()
    # print "End_time:", End_time
    print "Runtime:", "".join([str(End_time - Start_time), '(s)'])
    with open(options.outpath + '/runtime.json') as fp:
        runtime = json.load(fp, object_pairs_hook=OrderedDict)
    QC_total_time = QC_time_def(runtime)
    QC_threads = QC_thread_def(runtime)
    print "GVC: ", "Threads:", runtime['GVC']['thread'], ",Total_time:", "".join(
        [str(runtime['GVC']['total']), '(s)'])
    print "QC: ", "Threads:", round(sum(
        QC_threads) / len(QC_threads)), ",Total_time:", "".join([str(sum(QC_total_time)), '(s)'])
    # if os.path.exists(options.outpath + '/logFile'):
    #     with open(options.outpath + '/logFile') as f:
    #         lines = f.readlines()
    #         for line in lines:
    #             if 'ERROR' in line or 'Error' in line:
    #                 print line
    if os.path.exists(options.outpath + '/runtime.json'):
        os.remove(options.outpath + '/runtime.json')
    if os.path.exists(options.outpath + '/logFile'):
        os.remove(options.outpath + '/logFile')



from toil.common import Toil
from toil.job import Job

from runner.runner import docker_runner, decorator_wrapt
import re
import docker


class verify_license(Job):
    @decorator_wrapt
    def __init__(self, image, volumes, *args, **kwargs):
        self.image = image
        self.volumes = volumes
        self.commandLine = ['check_license']
        return super(verify_license, self).__init__(*args, **kwargs)

    @docker_runner("verify_license")
    def run(self, fileStore):
        return self.commandLine


def get_license_status(gvc_lib, image):
    """Get GVC product license status.

    Args:
        gvc_lib: gvc_lib abosulte path (str type)
        image:  docker images like  xgboost:1.0-41-g9f68680

    Return:
    {
        "returnCode": 0,  0 or 1 
        "error_file":"gvc.lic",   gvc.lic or license.txt
        "error_msg":"output"
    }
    """
    client = docker.from_env()
    volumes = {
        gvc_lib: {
            "bind": "/Genowis",
            "mode": "rw"
        }
    }
    contains = client.containers.run(
        image, ['check_license'], network_mode='host', volumes=volumes, detach=True)

    stream = contains.logs(stream=True)
    buffer = ''.join(stream)
    if contains.wait() != 0:
        if re.search('license ERROR :', buffer):
            contains.remove(v=True)
            print ([buffer])
            error_msg = re.search(
                '(?<=license ERROR :\n)\s+(.*)\s+(?=\nthe pc signature is :)', buffer,re.MULTILINE).group(1)
            return {"returnCode": -1,
                    "error_file": "gvc.lic",
                    "error_msg": error_msg
                    }
        else:
            contains.remove(v=True)
            error_msg = buffer
            return {"returnCode": -1,
                    "error_file": "license.txt",
                    "error_msg": buffer.strip('\n')
                    }
    else:
        contains.remove(v=True)
        return {"returnCode": 0}



#!/usr/bin/python
from __future__ import absolute_import
import json
import os
import sys
import shlex
import subprocess
from collections import OrderedDict

import docker
from toil.common import Toil
from toil.job import Job

from . import bam_load_check
from . import bed_check_sort_reg
from . import gvc_check
from . import gvc_pipeline
from . import qc_pipeline
from . import predict
from . import vcf_oncodecode
from . import license_job
from . import cnv
import pysam
import copy
import itertools
import argparse
# import variant_interface

import re


def remap_bam(ref, bam, thread, docker, volumes):
    # https://www.biostars.org/p/134638/
    # samtools fastq reads.bam | bwa mem -p ref.fa -
    oldbam = bam + '.old.bam'
    os.rename(bam, oldbam)
    os.rename(bam + '.bai', oldbam + '.bai')
    vol_str = ''
    for k in volumes.keys():
        vol_str = vol_str + ' -v ' + k + ':' + k + ' '
    # docker run -v /db:/db -v/disk:/disk bwa_bam_pipe:1.0-8-g09a12df sh -c \
    # "samtools fastq /disk/bam2vcf/data/T.bam.old.bam | bwa mem -t 12 -p \
    # /db/WEBtool/ref/human.fa -
    # | samtools view -@ 12  -b -T /db/WEBtool/ref/human.fa -S - \
    # | samtools sort -@ 12 - -o /disk/bam2vcf/data/T.bam
    # && samtools index /disk/bam2vcf/data/T.bam"
    cmd = ('docker run ' + vol_str + docker + ' sh -c "samtools fastq ' +
           oldbam + ' | bwa mem -t ' + thread + ' -p ' + ref + ' - ' +
           ' | samtools view -@ ' + thread + ' -b -T ' + ref + ' -S - ' +
           ' | samtools sort -@ ' + thread + ' - -o ' + bam +
           ' && samtools index ' + bam + '"')
    print(cmd)
    subprocess.call(shlex.split(cmd))
    return True


def cmd_parser(absPath):
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument(
        'input_json',
        help='The json file stores names and paths of both normal and tumor samples.'
        + '\n' +
        'eg: {"N": ["/disk/N.sort.dup.bam"], "T": ["/disk/T.sort.dup.bam"]}')
    parser.add_argument('reference', help='The reference fasta file')
    parser.add_argument('outpath', help='The output folder')
    parser.add_argument(
        '--dbsnp',
        help="The Single Nucleotide Polymorphism Database(dbSNP) file", required=True)
    parser.add_argument(
        '--bed',
        help="""BED file for WES or Panel analysis. It should be a TAB delimited file
with at least three columns: chrName, startPosition and endPostion""")
    parser.add_argument(
        '--segmentSize',
        help="Chromosome segment size for each GVC job, set to 100000000 (100MB) or larger for better performance. Default is to run only one GVC job.")
    parser.add_argument(
        '--gvc_lib',
        help="GVC library folder",
        required=True)

    parser.add_argument(
        '--region',
        help=argparse.SUPPRESS
    )
    parser.add_argument('--strategy', choices=['WES', 'WGS', 'Panel'],
                        help='Switch algorithm for WES, Panel or WGS analysis')
    parser.add_argument(
        '--mutantType',
        help="Switch algorithm for germline or somatic mutation detection",
        choices=['Somatic', 'Germline'],
        action='append',
        default=[])
    parser.add_argument('--CNV', action='store_true', default=False,
                        help='calculate and output CNV simultaneously')
    parser.add_argument('--SV', action='store_true', default=False,
                        help='calculate and output SV simultaneously')
    parser.add_argument(
        '--sample_name', help="Name of the sample to be analyzed.", default='sample_name')
    parser.add_argument('--rmtmp', action='store_true',
                        default=False, help='remove tempelate file')

    return parser.parse_args()


def check_dir_volume(name, paramter, file_path, msg=None):
    if not file_path:
        print("ERROR:{0} is None,please check paramter{1}.".format(
            name, paramter))
        sys.exit(-1)

    if not isinstance(file_path, str) and not isinstance(file_path, unicode):
        print (type(file_path))
        print("filepath is not string")
        sys.exit(-2)
    if os.path.exists(file_path) is False:
        print("ERROR:{0} file not found:{1}".format(name, file_path))
        if msg:
            print(msg)
        sys.exit(-3)


def check_images_exist(version):
    client = docker.from_env()
    for val in version.values():
        try:
            client.images.get(val)
        except docker.errors.ImageNotFound as E:
            print(val, "images can't found")
            sys.exit(4)
        except Exception as E:
            print(E)
            sys.exit(5)


def get_link_files(v_path):
    #print ("get_link_files:",v_path,os.path.realpath(v_path))
    if os.path.isfile(v_path) and os.path.realpath(v_path) != v_path:
        return [os.path.realpath(v_path)]
    elif os.path.realpath(v_path) != v_path:
        return []

    vpath_inner = [map(lambda x: os.path.join(root, x), files)
                   for root, _, files in os.walk(v_path)]
    return filter(lambda x: x != os.path.realpath(x), list(itertools.chain.from_iterable(vpath_inner)))


def get_dynamic_volumes(*intput_paths):
    paths = map(lambda x: os.path.abspath(re.sub('/+', '/', x)), intput_paths)
    links = map(get_link_files, paths)
    subpath = list(itertools.chain.from_iterable(links))
    subpath.extend(paths)
    dirpaths = map(lambda x: os.path.dirname(
        x) if os.path.isfile(x) else x, subpath)
    realpahts = map(os.path.realpath, dirpaths)
    dirpaths.extend(realpahts)
    dirpaths = map(lambda x: '/'.join(x.split('/')[:2]), dirpaths)
    bindpaths = map(lambda x: {'bind': x, 'mode': 'rw'}, dirpaths)
    return dict(zip(dirpaths, bindpaths))


def bam_vcf_pipeline(root, gvc_lib, reference, dbsnp, strategy, bed, outpath, group_info, mutantType, segmentSize,
                     sample_name, model_path, check_bam, enable_vcf_oncodecode=False, CNV_enable=True, SV_enable=True, rm_temp=True, volumes=None, limit_regions=None):
    variants = ['snv', 'indel', 'sv']
    gvc_varians = ['snv', 'indel', 'cnv', 'sv']
    filtermodel = ['grig', 'xgboost', 'filter']
    if not CNV_enable:
        gvc_varians.remove('cnv')

    if not SV_enable:
        variants.remove('sv')
        gvc_varians.remove('sv')

    # check version volumes file path
    version_path = gvc_lib + '/version.json'
    #volumes_path = gvc_lib + '/volumes.json'
    #thread_path = gvc_lib + '/thread.json'
    xml_path = gvc_lib + '/format.xml'

    #check_dir_volume("volumn", "--gvc_lib", volumes_path)
    check_dir_volume("version", "--gvc_lib", version_path)
    #check_dir_volume("thread", "--gvc_lib", thread_path)
    check_dir_volume("xml", "--gvc_lib", xml_path)
    check_dir_volume("Module", "--gvc_lib", model_path)

    with open(version_path, 'r') as verf:
        version = json.load(verf)

    # with open(thread_path, 'r') as thref:
    #     thread = json.load(thref)

    # check fa and fa.fai
    check_dir_volume("Reference ", "reference", reference)
    check_dir_volume("fai", "reference", reference + '.fai',
                     "please user samtools: samtools faidx {}".format(reference))

    # check docker images
    # check_images_exist(version)
    # check dbsnp dir
    check_dir_volume("dbsnp frequency", "--dbsnp", dbsnp)

    # check output dir and volumes dir
    check_dir_volume("outpath", "outpath", outpath)
    # check bed dir
    if strategy == 'WES' or strategy == 'Panel':
        check_dir_volume("bed", "--bed", bed)
        bed_sort_file = outpath + '/' + os.path.basename(bed) + '.sort'
        bed_check_sort_reg.bed_check_sort_reg(reference, bed, bed_sort_file)
    # tmp dir
    temp = outpath + '/tmp/'
    if os.path.exists(temp) is False:
        os.makedirs(temp)
    # Bam list
    for skey in group_info.keys():
        if len(group_info[skey]) > 1:
            print("ERROR: At current version, we only support one bam file " +
                  "for tumor sample and one bam file for normal sample.")
            sys.exit(1)
    if len(group_info) == 1 or len(group_info['N'][0]) == 0:
        if len(group_info['T'][0]) > 1:
            bam_list = group_info['T']
            group_info = {'T': group_info['T']}
            if 'Somatic' in mutantType:
                print(
                    "ERROR: At current version, we only support Germline for Single-sample.")
                sys.exit(1)
            mutantType = ['Germline']
        else:
            print("ERROR: The json file format is error. the format is " +
                  '{"T": ["/disk/T.sort.dup.bam"]}')
            sys.exit(1)

    if mutantType != ['Germline']:
        bam_list = group_info['N'] + group_info['T']
        group_info = OrderedDict(zip(['N', 'T'], bam_list))
    else:
        bam_list = group_info['T']
        group_info = {'T': group_info['T']}

    bam_list = map(os.path.abspath, bam_list)

    if not volumes:
        if bed:
            volumes = get_dynamic_volumes(
                reference, model_path, gvc_lib, dbsnp, bed, outpath, *bam_list)
        else:
            volumes = get_dynamic_volumes(
                reference, model_path, gvc_lib, dbsnp, outpath, *bam_list)
        # print(gvc_lib)
        #print ("*******volumes:",volumes)
        # sys.exit(-1)
    if check_bam:
        if mutantType == ['Germline']:
            check_dir_volume("bam", "input_json", group_info['T'][0])
        else:
            check_dir_volume("bam", "input_json", group_info['N'][0])
            check_dir_volume("bam", "input_json", group_info['T'][0])

        for bam in bam_list:
            bam_load_check.bamBai(bam)
            bam_ref_logical = bam_load_check.bamRefCheck(reference,
                                                         bam)
            if bam_ref_logical is False:
                print ('Chromosome name in bam file is different from reference.\
                     Check {} for correct chromosome names.'.format(bam))
                sys.exit(-1)

    if strategy == 'WGS':
        bed_sort_file = None

    gvc_volumes = copy.deepcopy(volumes)
    gvc_volumes[gvc_lib] = {
        "bind": "/Genowis",
        "mode": "rw"
    }
    check_license_job = license_job.verify_license(
        version['xgboost'], gvc_volumes)
    root.addChild(check_license_job)
    root = root.encapsulate()
    feature_dict = gvc_pipeline.gvc_pipeline(
        root,
        reference,
        segmentSize,
        bam_list,
        temp,
        sample_name,
        version,
        gvc_volumes,
        gvc_varians,
        rm_gvcs=True,
        dbsnp=dbsnp,
        ccd=bed_sort_file,
        limit_regions=limit_regions)
    root = root.encapsulate()
    #print (group_info.keys(),bam_list)

    if CNV_enable and 'Somatic' in mutantType:
        new_volumes = copy.deepcopy(volumes)
        new_volumes[os.path.join(gvc_lib, 'database', 'CNV')] = {
            "bind": "/disk/software/GVC-CNV/",
            "mode": "rw"
        }
        cnv_job = cnv.cnv(sample_name, os.path.basename(
            feature_dict['cnv'][0]), os.path.basename(feature_dict['cnv'][1]), outpath, group_info.keys(), os.path.dirname(feature_dict['cnv'][0]), version['cnv'], new_volumes, enable_panel=strategy == 'Panel')
        root.addChild(cnv_job)

    if strategy == 'Panel':
        strategy = "Panel_tissue"
    seqtype = "Hiseq"
    if strategy == 'WGS':
        if os.path.exists(reference + '.reg') is False:
            qc_pipeline.rootUpdateJob_pipeline(
                root, reference, None, strategy, version, volumes)
            root = root.encapsulate()

    qc_txts = qc_pipeline.qc_pipeline(root, temp, outpath, strategy, bam_list, group_info.keys(
    ), reference, bed_sort_file, sample_name, version, volumes)

    root = root.encapsulate()
    setting_dict = qc_pipeline.qc2predict(
        root, qc_txts, temp, group_info.keys(), seqtype, strategy,
        variants, mutantType, version, volumes)

    predict_volumes = copy.deepcopy(volumes)
    predict_volumes[gvc_lib] = {
        "bind": "/Genowis",
        "mode": "rw"
    }

    root = root.encapsulate()
    # predict
    if strategy == 'WGS':
        predict_dict = predict.Predict_pipeline(root, sample_name,
                                                group_info.keys(), outpath,
                                                setting_dict, feature_dict, variants, seqtype,
                                                strategy, mutantType, None,
                                                filtermodel, xml_path, model_path, bam_list[-1], version,
                                                predict_volumes)
    elif strategy == 'WES' or strategy == "Panel_tissue":
        predict_dict = predict.Predict_pipeline(root, sample_name,
                                                group_info.keys(), outpath,
                                                setting_dict, feature_dict, variants, seqtype,
                                                strategy, mutantType, bed_sort_file,
                                                filtermodel, xml_path, model_path, bam_list[-1], version,
                                                predict_volumes)
    root = root.encapsulate()
    if 'Somatic' in mutantType:
        predict.post_filter(root, predict_dict, [
                            'snv', 'indel'], reference, bam_list[-1], sample_name, outpath, version, volumes)
        root = root.encapsulate()
    # vcf_oncodecode
    if enable_vcf_oncodecode:
        vcf_oncodecode.run_vcf_oncodecode(
            root, outpath, sample_name, variants, filtermodel,
            mutantType, strategy, version, volumes)
        root = root.encapsulate()
    # rm predict tmp

    if rm_temp:
        if strategy in ('WES', "Panel_tissue"):
            predict.tmp_rm_def(
                root,
                sample_name,
                outpath,
                variants,
                filtermodel,
                strategy,
                mutantType,
                rm_predict=True, bed=bed)
        else:
            predict.tmp_rm_def(
                root,
                sample_name,
                outpath,
                variants,
                filtermodel,
                strategy,
                mutantType,
                rm_predict=True, bed=None)
    return root


def get_bam_lengths(group_info):
    choice_length = [100, 150]
    limit = 10
    l_arv_reads_all_bams = list()
    for bams in group_info.values():
        for bam in bams:
            l_reads_bam = list()
            primary_reads = pysam.AlignmentFile(bam)
            for reads in primary_reads.fetch():
                if len(l_reads_bam) > 10:
                    break
                if (reads.flag & (pysam.FSECONDARY | pysam.FSUPPLEMENTARY)) > 0 and reads.flag & pysam.FUNMAP == 0:
                    l_reads_bam.append(reads.infer_read_length())

            l_arv_reads_all_bams.append(sum(l_reads_bam)/len(l_reads_bam))
            primary_reads.close()
    arv_len = sum(l_arv_reads_all_bams) / len(l_arv_reads_all_bams)
    return min(choice_length, key=lambda x: abs(x-arv_len))


def get_model_path(gvc_lib_path, choice_length):
    return '{}/Model/PE{}'.format(gvc_lib_path, choice_length)


if __name__ == "__main__":

    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    options = cmd_parser(absPath)
    options.stats = True

    license_volum = {
        options.gvc_lib: {"bind": "/Genowis",
                                  "mode": "rw"}
    }
    start_job = Job()
    # load bam
    check_dir_volume('input_json', 'input_json', options.input_json)

    limit_regions = gvc_pipeline.get_regions_file(options.region)
    with open(options.input_json) as fp:
        group_info = json.load(fp, object_pairs_hook=OrderedDict)

    model_length = get_bam_lengths(group_info)
    model_path = get_model_path(options.gvc_lib, model_length)
    start_job = bam_vcf_pipeline(start_job, options.gvc_lib, options.reference, options.dbsnp, options.strategy, options.bed,
                                 options.outpath, group_info, options.mutantType, options.segmentSize,
                                 options.sample_name, model_path, check_bam=True, CNV_enable=options.CNV, SV_enable=options.SV, rm_temp=options.rmtmp,limit_regions=limit_regions)
    #vd = variant_interface.VariantFeature()
    #print (vd.get_feature('snv'))
    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()



import unittest
from . import gvc_vcf_pipeline



class TestDbsnp(unittest.TestCase):
    

    def test_correct_filename(self):
        gvc_vcf_pipeline.check_dbsnp_format("/disk/db/ref//human.fa",'/disk/db/dbsnp/dbsnp_138-1000G-snp.RS-1000G.1-Y.sort.nonchr')
        
if __name__ == "__main__":
    unittest.main()


import json
import os
import sys
from StringIO import StringIO
import shlex
import subprocess
import time
import argparse
import re
import collections


def add_time(toiltime, walltime, key):
    ret = dict()
    if key in toiltime['job_types'].keys():
        ret['thread'] = toiltime['job_types'][key]['total_number']
        ret['avg'] = toiltime['job_types'][key]['average_time']
        ret['max'] = toiltime['job_types'][key]['max_time']
        ret['total'] = toiltime['job_types'][key]['total_time']
    if (key in walltime.keys() and 'Start' in walltime[key].keys()
            and 'End' in walltime[key].keys()):
        ret['start'] = int(walltime[key]['Start'])
        ret['end'] = int(walltime[key]['End'])
        ret['walltime'] = int(walltime[key]['End']) - int(
            walltime[key]['Start'])
    return {key: ret}


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description='Check Toil run jobStore and output runtime JSON file')
    parser.add_argument(
        'inputLog',
        help="STDERR output from Toil run, with Job Start/End Info.")
    parser.add_argument('jobStore', help="Toil run jobStore pathname.")
    parser.add_argument('outputJSON', help="output JSON file.")
    parser.add_argument(
        '-k', '--keep', action='store_true', help="keep jobStore undeleted.")
    args = parser.parse_args()

    reportJobNames = [
        'GVC', 'LinearPredict', 'XGBoostPredict', 'BedFilter', 'GenerateVCF',
        'QC', 'QC_Postprocess',
        'Annotation'
    ]  # all jobNames you want to see in output JSON file

    walltime = collections.defaultdict(dict)
    rex = re.compile(r'Runtime\.([^\.]+)\.(Start|End)=(\d+)')
    with open(args.inputLog, 'r') as infile:
        for line in infile:
            matchTime = rex.search(line)
            if matchTime:
                (jobName, typeStr, timestamp) = matchTime.group(1, 2, 3)

                if (not typeStr in walltime[jobName].keys()
                        or (typeStr == 'Start'
                            and timestamp < walltime[jobName][typeStr])
                        or (typeStr == 'End'
                            and timestamp > walltime[jobName][typeStr])):
                    walltime[jobName][typeStr] = timestamp

    # get runtime from jobStore
    stats_cmd = 'toil stats --raw file:' + args.jobStore
    p = subprocess.Popen(
        shlex.split(stats_cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    (log, err) = p.communicate()
    if p.returncode != 0:
        sys.stderr.write('ERROR: Cloud not get runtime from jobStore.')
        sys.stderr.write(err)
        exit(1)
    # parse runtime then write our own runtime.json
    toiltime = json.load(StringIO(log))
    runtime = dict()
    # use the earliest job start time as analysis_start_time
    runtime['analysis_start_time'] = 0  # init
    runtime['analysis_runtime'] = toiltime['total_run_time']
    for jobName in reportJobNames:
        runtime.update(add_time(toiltime, walltime,
                                jobName))  # info from jobStore and INFO
        if ('Start' in walltime[jobName].keys() and
            (runtime['analysis_start_time'] == 0
             or runtime['analysis_start_time'] > walltime[jobName]['Start'])):
            runtime['analysis_start_time'] = walltime[jobName]['Start']

    timeJsonFile = args.outputJSON
    with open(timeJsonFile, 'w') as outfile:
        json.dump(runtime, outfile, indent=4)

    # clean jobstore
    if not args.keep:
        subprocess.call(['toil', 'clean', args.jobStore])



# -*- coding: UTF-8 -*-

from setuptools import setup, find_packages
import os
# or
# from distutils.core import setup

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README')) as f:
    README = f.read()

setup(
    name='gvc4bam',     # 包名字
    version='1.0.9',   # 包版本
    description='GVC4BAM is a data processing pipeline developed by Genome Wisdom Inc. GVC4FASTQ detects germline and somatic mutations (SNV, InDel, SV) from BAM files.',   # 简单描述
    author='LongHui.Yin',  # 作者
    author_email='dragonfly.yin@genowis.com',  # 作者邮箱
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(),                 # 包
    #Programming Language :: Python :: 2.7
    classifiers=['Programming Language :: Python :: 2.7','License :: Free For Educational Use'],
    install_requires=['docker==2.5.1', 'pysam', 'toil', 'toil-runner'],
    scripts=['tools/bam_vcf.py','tools/check_runtime.py']
)



import unittest
import gvc_vcf_pipeline

# NO time to Generate bam to test, so specify train1 test data


class TEST_BAM_LENGTH(unittest.TestCase):
    def test_Germline_100bp(self):
        group_infos = {'T': [
            '/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Germline/SRR1611178/T.sort.dup.bam']
        }
        self.assertEqual(100, gvc_vcf_pipeline.get_bam_lengths(group_infos))
    # Germline

    def test_Somatic_100bp(self):
        group_infos = {'T': [
            '/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Germline/SRR1611178/T.sort.dup.bam'],
            'N': ['/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Germline/SRR1611178/T.sort.dup.bam']
        }
        self.assertEqual(100, gvc_vcf_pipeline.get_bam_lengths(group_infos))

    def test_Germline_150bp(self):
        group_infos = {
            'T': ['/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Somatic/WES/Ct180554/N.sort.dup.bam']}

        self.assertEqual(150, gvc_vcf_pipeline.get_bam_lengths(group_infos))

    def test_Somatic_150bp(self):
        group_infos = {
            'N': ['/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Somatic/WES/Ct180554/N.sort.dup.bam'],
            'T': ['/disk/gvcgroup/fushun/20190227_Fq2VcfDockerTest/result/Somatic/WES/Ct180554/T.sort.dup.bam']}
        self.assertEqual(150, gvc_vcf_pipeline.get_bam_lengths(group_infos))

    def test_model_150(self):
        self.assertEqual('/disk/yinlh/docker3.0/gvc_lib//Model/PE150',
                         gvc_vcf_pipeline.get_model_path('/disk/yinlh/docker3.0/gvc_lib/', 150))

    def test_model_100(self):
        self.assertEqual('/disk/yinlh/docker3.0/gvc_lib//Model/PE100',
                         gvc_vcf_pipeline.get_model_path('/disk/yinlh/docker3.0/gvc_lib/', 100))


if __name__ == "__main__":
    unittest.main()



from toil.common import Toil
from toil.job import Job
from collections import OrderedDict

import pwd
import getpass
import argparse
import os
import sys
import json
import docker
from runner.runner import docker_runner, decorator_wrapt


class GVC_check(Job):
    @decorator_wrapt
    def __init__(self, image, volumes, *args, **kwargs):
        self.image = image
        self.volumes = volumes

        self.commandLine = ['/usr/local/bin/gvcR']
        super(GVC_check, self).__init__(*args, **kwargs)

    @docker_runner('gvc_check')
    def run(self, fileStore):
        return self.commandLine
        # client = docker.from_env()
        # client.containers.run(
        #     self.image, self.commandLine, volumes=self.volumes, privileged=True, remove=True, user=pwd.getpwnam(getpass.getuser()).pw_uid)


def cmd_parser(absPath):
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument('--version', type=argparse.FileType('r'),
                        default=file(absPath + '/../etc/version.json'))
    parser.add_argument('--volumes', type=argparse.FileType('r'),
                        default=absPath + '/../etc/volumes.json')
    return parser.parse_args()


def gvc_check_fun(root, version, volumes):
    job = GVC_check(version['gvc_check'], volumes)
    root.addChild(job)


if __name__ == "__main__":
    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    options = cmd_parser(absPath)

    version = json.load(options.version, object_pairs_hook=OrderedDict)
    volumes = json.load(options.volumes, object_pairs_hook=OrderedDict)
    start_job = Job()
    gvc_check_fun(start_job, version, volumes)

    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()




import shutil
import re
from toil.common import Toil
from toil.job import Job
import docker

import pwd
import getpass
import sys
import os
import itertools
import argparse
import json
import time

from collections import defaultdict, OrderedDict
from runner.runner import docker_runner, decorator_wrapt, docker_runner_method


class QC_DefineRegion(Job):
    @decorator_wrapt
    def __init__(self, ref, bed, strategy, image, volumes, *args, **kwargs):
        self.ref = ref
        self.bed = bed
        self.strategy = strategy
        self.image = image
        self.volumes = volumes
        if strategy == 'WGS':
            self.commandLine = [
                'python', '/usr/local/bin/rootUpdate.py', self.ref, '--seqtype', self.strategy]
        elif strategy == 'WES' or strategy == 'Panel_tissue':
            self.commandLine = ['python', '/usr/local/bin/rootUpdate.py',
                                self.ref, '--input_bed', self.bed, '--seqtype', self.strategy]
        super(QC_DefineRegion, self).__init__(*args, **kwargs)

    @docker_runner("QC_DefineRegion")
    def run(self, fileStore):
        return self.commandLine


def rootUpdateJob_pipeline(root, ref, bed, strategy, version, volumes):
    rootUpdate_job = QC_DefineRegion(
        ref, bed, strategy, version['gvcqc'], volumes)
    root.addChild(rootUpdate_job)


class QC(Job):
    @decorator_wrapt
    def __init__(self, volumes, image, config, *args, **kwargs):
        self.volumes = volumes
        self.image = image
        self.command = ['bam2qc.py', config]
        super(QC, self).__init__(*args, **kwargs)

    @docker_runner('QC')
    def run(self, fileStore):
        return self.command


def qc_config(output_name, strategy, l_bam_path, l_bam_names, reference, bed, sample_name):

    config_dict = dict()
    if not isinstance(l_bam_names, list):
        raise ValueError("l_bam_name must be list type")
    config_dict['bam_name'] = l_bam_names
    config_dict['sample_id'] = sample_name

    if not isinstance(l_bam_path, list):
        raise ValueError("l_bam_path must be slist type")
    config_dict['bam'] = l_bam_path
    if re.search('panel', strategy, re.I) or strategy == 'WES':
        config_dict['bed'] = bed
    else:
        config_dict['reference'] = reference
    config_dict['outpath'] = output_name
    config_dict['enable_error_flags'] = "False"
    return config_dict


def qc_pipeline(root, tmp_path, output_name, strategy, l_bam_path, l_bam_names, reference, bed, sample_name, version, volumes):
    config_dict = qc_config(output_name, strategy, l_bam_path,
                            l_bam_names, reference, bed, sample_name)
    qc_config_file = os.path.join(tmp_path, 'qc_config.json')
    with open(qc_config_file, 'w') as config_f:
        json.dump(config_dict, config_f)
    qcjob = QC(volumes,
               version['gvcqc'], qc_config_file,cores=3)
    root.addChild(qcjob)
    return map(lambda x: os.path.join(output_name, x, 'QC.txt'), l_bam_names)


class QC_Postprocess(Job):
    @decorator_wrapt
    def __init__(self, txt_list, outfile, sequenceType, strategy, variant, mutantType, sample_name, image, volumes, *args, **kwargs):
        self.commandLine = ['predictJson.py']
        self.commandLine.extend(txt_list)
        self.commandLine.extend(
            [outfile, sequenceType, strategy, variant, mutantType, ','.join(sample_name)])
        self.image = image
        self.volumes = volumes
        super(QC_Postprocess, self).__init__(*args, **kwargs)

    @docker_runner("QC_Postprocess")
    def run(self, fileStore):
        return self.commandLine


class rmQCtemp(Job):
    def __init__(self, l_dirs, *args, **kwargs):
        self.l_dirs = map(os.path.dirname, l_dirs)
        super(rmQCtemp, self).__init__(*args, **kwargs)

    def run(self, fileStore):
        for dir_name in self.l_dirs:
            shutil.rmtree(dir_name)


def qc2predict(root, qc_txts, output_path, samples, sequenceType, strategy, variants, mutantTypes, version, volumes):
    json_dict = defaultdict(type(defaultdict(str)))
    for variant, mutantType in itertools.product(variants, mutantTypes):
        outfile = output_path + '/' + \
            '.'.join([sequenceType, strategy, variant, mutantType, 'json'])
        json_dict[variant][mutantType] = outfile
        jsonJob = QC_Postprocess(qc_txts, outfile, sequenceType,
                                 strategy, variant, mutantType, samples, version['gvcqc'], volumes)
        root.addChild(jsonJob)
    #rmQC = rmQCtemp(qc_txts)
    #root.addFollowOn(rmQC)
    return json_dict


def cmdPhaser(absPath):
    parser = Job.Runner.getDefaultArgumentParser()
    parser.add_argument("tmp", help="tmp path")
    parser.add_argument('output_path', help="output path")
    parser.add_argument('bam', nargs='+', help="bam")
    parser.add_argument("sample_name", help="sample name")
    parser.add_argument(
        '--strategy', choices=['WES', 'WGS', 'Panel_tissue'])
    parser.add_argument("--bamname", action='append', help="bam name")
    parser.add_argument(
        '--reference', help="input reference file"
    )
    parser.add_argument(
        "--bed", help="bed file"
    )
    parser.add_argument(
        '--seqtype', choices=['Hiseq', 'CN500', 'Novaseq'], default='Hiseq')
    parser.add_argument(
        '--filtermodel', choices=['xgboost', 'filter', 'empirical', 'grig'], metavar='model', action='append', default=[])
    parser.add_argument(
        '--mutantType', choices=['Somatic', 'Germline'], action='append', default=[])
    parser.add_argument(
        '--version', type=argparse.FileType('r'), required=True)
    parser.add_argument(
        '--volumes', type=argparse.FileType('r'), required=True)

    return parser.parse_args()


if __name__ == "__main__":
    absPath = os.path.abspath(os.path.dirname(sys.argv[0]))
    options = cmdPhaser(absPath)
    version = json.load(options.version)
    volumes = json.load(options.volumes)
    start_job = Job()

    qc_txts = qc_pipeline(start_job, options.tmp, options.output_path, options.strategy,
                          options.bam, options.bamname,
                          options.reference, options.bed, options.sample_name, version, volumes)

    start_job = start_job.encapsulate()

    variants = ['snv', 'indel']
    qc2predict(start_job, qc_txts, options.output_path, options.sample_name, options.seqtype, options.strategy,
               variants, options.mutantType, version, volumes)

    with Toil(options) as toil:
        if not toil.options.restart:
            toil.start(start_job)
        else:
            toil.restart()
